@@ -3,7 +3,7 @@
 # Copyright (c) 2003, Paul McGuire
 #
 
-from pyparsing import Word, Optional, OneOrMore, Group, ParseException
+from pyparsing import Word, Optional, OneOrMore, Group, ParseException, Regex
 
 atomicWeight = {
     "O"  : 15.9994,
@@ -23,22 +23,24 @@ def test( bnf, strg, fn=None ):
             print fn( bnf.parseString( strg ) )
         else:
             print
-    
-caps = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
-lowers = caps.lower()
+
 digits = "0123456789"
 
-element = Word( caps, lowers )
+# Version 1
+element = Regex("A[cglmrstu]|B[aehikr]?|C[adeflmorsu]?|D[bsy]|"
+                "E[rsu]|F[emr]?|G[ade]|H[efgos]?|I[nr]?|Kr?|L[airu]|"
+                "M[dgnot]|N[abdeiop]?|Os?|P[abdmortu]?|R[abefghnu]|"
+                "S[bcegimnr]?|T[abcehilm]|Uu[bhopqst]|U|V|W|Xe|Yb?|Z[nr]")
 elementRef = Group( element + Optional( Word( digits ), default="1" ) )
 formula = OneOrMore( elementRef )
 
-fn = lambda x : "(" + str(sum( [ atomicWeight[elem]*int(qty) for elem,qty in x ] ) ) + ")"
+fn = lambda elemList : sum( [ atomicWeight[elem]*int(qty) for elem,qty in elemList ] )
 test( formula, "H2O", fn )
 test( formula, "C6H5OH", fn )
 test( formula, "NaCl", fn )
+print
 
-
-element = Word( caps, lowers )
+# Version 2 - access parsed items by field name
 elementRef = Group( element.setResultsName("symbol") + \
                 Optional( Word( digits ), default="1" ).setResultsName("qty") )
 formula = OneOrMore( elementRef )
@@ -47,7 +49,18 @@ fn = lambda elemList : sum( [ atomicWeight[elem.symbol]*int(elem.qty) for elem i
 test( formula, "H2O", fn )
 test( formula, "C6H5OH", fn )
 test( formula, "NaCl", fn )
+print
 
+# Version 3 - convert integers during parsing process
+integer = Word( digits ).setParseAction(lambda t:int(t[0]))
+elementRef = Group( element.setResultsName("symbol") + \
+                Optional( integer, default=1 ).setResultsName("qty") )
+formula = OneOrMore( elementRef )
+
+fn = lambda elemList : sum( [ atomicWeight[elem.symbol]*elem.qty for elem in elemList ] )
+test( formula, "H2O", fn )
+test( formula, "C6H5OH", fn )
+test( formula, "NaCl", fn )
     
 
 
