@@ -13,11 +13,5 @@ anchor = anchorStart + SkipTo(anchorEnd).setResultsName("body") + anchorEnd
 
 
 for tokens,start,end in anchor.scanString(htmlText):
-    print tokens
-    tokens = dict(tokens)
-    print tokens.keys()
-    body = tokens.pop('body','LSJDF')
-    print tokens.keys()
-    print
-    break
+    print tokens.body,'->',tokens.href
     
\ No newline at end of file
